MaybeEither

Type Alias MaybeEither 

pub type MaybeEither<L, R> = Option<Either<L, R>>;
A shortcut to represent the possibility of neither left nor right values existing by wrapping the Either<L, R> in an Option.

Aliased Type

pub enum MaybeEither<L, R> {
    None,
    Some(Either<L, R>),
}

Variants

1.0.0

None

No value.

1.0.0

Some(Either<L, R>)

Some value of type T.